# Copyright (c) 2011-2013 Cloudera, Inc. All rights reserved.

import httplib
import socket
import sys
import urllib2
from urllib2_kerberos import HTTPKerberosAuthHandler

# urlopen_with_timeout.
# The optional secure_http_service_name parameter allows callers
# to connect to secure HTTP servers via the urllib2_kerberos library.
# We have a modified version of the HTTPKerberosAuthHandler code
# which takes the Kerberos service name rather than construct the
# name using the HTTP request host. We always add the
# HTTPKerberosAuthHandler to urllib2 opener handlers because it has
# no effect if security is not actually enabled.

def urlopen_with_timeout(url,
                         data=None,
                         timeout=None,
                         secure_http_service_name=None,
                         username=None,
                         password=None):

  if sys.version_info < (2, 6):
    # The timeout parameter to urlopen was introduced in Python 2.6.
    # To workaround it in older versions of python, we copy, with
    # minor modification, httplib.HTTPConnection, and hook it all
    # up.
    opener = urllib2.build_opener(
        HTTPKerberosAuthHandler(secure_http_service_name),
        _make_http_digest_auth_handler(url, username, password),
        _make_timeout_handler(timeout))
    return opener.open(url, data)
  else:
    opener = urllib2.build_opener(
        HTTPKerberosAuthHandler(secure_http_service_name),
        _make_http_digest_auth_handler(url, username, password))
    return opener.open(url, data, timeout)


def _make_timeout_handler(timeout):
  # Create these two helper classes fresh each time, since
  # timeout needs to be in the closure.

  class TimeoutHTTPConnection(httplib.HTTPConnection):
    def connect(self):
      """Connect to the host and port specified in __init__."""
      msg = "getaddrinfo returns an empty list"
      for res in socket.getaddrinfo(self.host, self.port, 0,
                      socket.SOCK_STREAM):
        af, socktype, proto, canonname, sa = res
        try:
          self.sock = socket.socket(af, socktype, proto)
          if timeout is not None:
            self.sock.settimeout(timeout)
          if self.debuglevel > 0:
            print "connect: (%s, %s)" % (self.host, self.port)
          self.sock.connect(sa)
        except socket.error, msg:
          if self.debuglevel > 0:
            print 'connect fail:', (self.host, self.port)
          if self.sock:
            self.sock.close()
          self.sock = None
          continue
        break
      if not self.sock:
        raise socket.error, msg

  class TimeoutHTTPHandler(urllib2.HTTPHandler):
    http_request = urllib2.AbstractHTTPHandler.do_request_
    def http_open(self, req):
      return self.do_open(TimeoutHTTPConnection, req)

  return TimeoutHTTPHandler

def _make_http_digest_auth_handler(url, username, password):
  password_manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
  password_manager.add_password(None, # realm
                                url,
                                username,
                                password)
  return urllib2.HTTPDigestAuthHandler(password_manager)
