-- Copyright (c) 2012 Cloudera, Inc. All rights reserved.
--
-- Version 4511 of the CMF schema.
--
-- Replaces role types in the config table with role config groups,
-- creating new groups as needed.
--

update SCHEMA_VERSION set VERSION=4511;

-- This is the new column; we'll remove the old ROLE_TYPE column
-- when we're done.
alter table CONFIGS
  add ROLE_CONFIG_GROUP_ID number;
alter table CONFIGS
  add constraint FK_CONFIG_ROLE_CONFIG_GROUP
  foreign key (ROLE_CONFIG_GROUP_ID)
  references ROLE_CONFIG_GROUPS (ROLE_CONFIG_GROUP_ID)
  deferrable initially immediate;

create index IDX_CONFIG_ROLE_CONFIG_GROUP on CONFIGS (ROLE_CONFIG_GROUP_ID);

-- Insert new groups for each service/roletype combination (from existing
-- configs) that's not already represented by a group.
--
-- Unlike the insert in schema 04509, we have to only include roletype
-- configs, hence the extra constraints on 'where'.
insert into ROLE_CONFIG_GROUPS (ROLE_CONFIG_GROUP_ID, SERVICE_ID, ROLE_TYPE, NAME)
  select LAST_GROUP.MAX_ID + rownum ROLE_CONFIG_GROUP_ID, inner_query.*
  from
    (select distinct c.SERVICE_ID, c.ROLE_TYPE, c.ROLE_TYPE || '_BASE'
     from CONFIGS c left join ROLE_CONFIG_GROUPS g
     on c.SERVICE_ID=g.SERVICE_ID and c.ROLE_TYPE=g.ROLE_TYPE
     where c.SERVICE_ID is not null and
           c.ROLE_TYPE is not null and
           g.SERVICE_ID is null) inner_query,
    (select nvl(max(ROLE_CONFIG_GROUP_ID), 0) MAX_ID
     from ROLE_CONFIG_GROUPS) LAST_GROUP;

-- Update all role type configs to point to the appropriate role config group.
update CONFIGS
  set ROLE_CONFIG_GROUP_ID=(
    select ROLE_CONFIG_GROUP_ID
    from ROLE_CONFIG_GROUPS g
    where CONFIGS.ROLE_TYPE=g.ROLE_TYPE and
          CONFIGS.SERVICE_ID=g.SERVICE_ID)
  where CONFIGS.SERVICE_ID is not null and
        CONFIGS.ROLE_TYPE is not null;

-- The ROLE_CONFIG_GROUP_ID column is fully populated so now we can remove
-- the ROLE_TYPE column.
alter table CONFIGS
  drop column ROLE_TYPE;
